/*
 * Copyright (c) 2009 by Internet Systems Consortium, Inc. ("ISC")
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* Import. */

#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <ncap.h>
#include <nmsg.h>
#include <nmsg/isc/nmsgpb_isc_ncap.h>

static nmsg_output_t output;
static nmsg_pbmod_t mod;

static int warned_unsupported;

static void callback(ncap_t, void *, struct ncap_msg *, const char *);

static void
callback(ncap_t input, void *user, struct ncap_msg *ncap, const char *label) {
	Nmsg__Isc__Ncap newncap;
	Nmsg__NmsgPayload *np;
	char dstip[16];
	char srcip[16];
	nmsg_res res;
	unsigned addrlen = 0;

	memset(&newncap, 0, sizeof(newncap));
	newncap.base.descriptor = mod->pbdescr;

	newncap.type = NMSG__ISC__NCAP_TYPE__Legacy;

	switch (ncap->np) {
	case ncap_ip4:
		memcpy(srcip, &ncap->npu.ip4.src.s_addr, 4);
		memcpy(dstip, &ncap->npu.ip4.dst.s_addr, 4);
		addrlen = 4;
		break;
	case ncap_ip6:
		memcpy(srcip, ncap->npu.ip6.src.s6_addr, 16);
		memcpy(dstip, ncap->npu.ip6.dst.s6_addr, 16);
		addrlen = 16;
		break;
	default:
		/* not reached */
		assert(0);
	}
	newncap.srcip.data = (uint8_t *) srcip;
	newncap.dstip.data = (uint8_t *) dstip;
	newncap.srcip.len = addrlen;
	newncap.dstip.len = addrlen;
	newncap.has_srcip = 1;
	newncap.has_dstip = 1;

	switch (ncap->tp) {
	case ncap_udp:
		newncap.lint0 = ncap->tpu.udp.sport;
		newncap.lint1 = ncap->tpu.udp.dport;
		newncap.has_lint0 = 1;
		newncap.has_lint1 = 1;
		break;
	case ncap_tcp:
	case ncap_icmp:
		if (warned_unsupported == 0) {
			fprintf(stderr, "ncap_tcp and ncap_icmp not supported\n");
			warned_unsupported = 1;
		}
		return;
		break;
	}

	newncap.payload.data = (uint8_t *) ncap->payload;
	newncap.payload.len = ncap->paylen;

	np = nmsg_payload_from_message(&newncap, NMSG_VENDOR_ISC_ID,
				       MSGTYPE_NCAP_ID, &ncap->ts);
	if (np == NULL) {
		fprintf(stderr, "nmsg_payload_from_message() failed\n");
		return;
	}

	res = nmsg_output_write(output, np);
	assert(res != nmsg_res_failure);
}

int
main(int argc, char **argv) {
	int in;
	int out;
	ncap_t input;
	nmsg_pbmodset_t ms;
	nmsg_res res;
	void *clos;

	/* initialize input and output file descriptors */
	if (argc == 3) {
		if (strcmp(argv[1], "-") == 0) {
			in = STDIN_FILENO;
		} else {
			in = open(argv[1], O_RDONLY);
			if (in == -1) {
				perror("open");
				return (EXIT_FAILURE);
			}
		}

		if (strcmp(argv[1], "-") == 0) {
			out = STDOUT_FILENO;
		} else {
			out = open(argv[2], O_WRONLY | O_CREAT | O_TRUNC, 0644);
			if (out == -1) {
				perror("open");
				return (EXIT_FAILURE);
			}
		}
	} else if (argc == 1) {
		in = STDIN_FILENO;
		out = STDOUT_FILENO;
	} else {
		fprintf(stderr, "usage: %s [<NCAPfile> <NMSGfile>]\n", argv[0]);
		return (EXIT_FAILURE);
	}

	/* create ncap object */
	input = ncap_create(70000);
	if (input == NULL) {
		perror("ncap_create");
		return (EXIT_FAILURE);
	}

	/* add ncap file input */
	input->add_nf(input, in, "");

	/* create nmsg output */
	output = nmsg_output_open_file(out, NMSG_WBUFSZ_MAX);
	if (output == NULL) {
		fprintf(stderr, "nmsg_output_open_file() failed\n");
		return (EXIT_FAILURE);
	}
	nmsg_output_set_zlibout(output, 1);

	/* load nmsg modules */
	ms = nmsg_pbmodset_init(NULL, 0);
	if (ms == NULL) {
		fprintf(stderr, "nmsg_pbmodset_init() failed\n");
		return (EXIT_FAILURE);
	}

	/* open handle to the nmsg ncap module */
	mod = nmsg_pbmodset_lookup(ms, NMSG_VENDOR_ISC_ID, MSGTYPE_NCAP_ID);
	if (mod == NULL) {
		fprintf(stderr, "nmsg_pbmodset_lookup() failed\n");
		return (EXIT_FAILURE);
	}

	/* initialize module */
	res = nmsg_pbmod_init(mod, &clos);
	if (res != nmsg_res_success)
		exit(res);

	/* ncap loop */
	input->collect(input, 0, callback, NULL);

	/* clean up */
	nmsg_output_close(&output);
	nmsg_pbmod_fini(mod, &clos);
	nmsg_pbmodset_destroy(&ms);
	input->destroy(input);

	return (EXIT_SUCCESS);
}
